from django.db import models
from polymorphic.models import PolymorphicModel
from django.contrib.auth.models import User, Group, Permission
from .managers import StateManager

class State(PolymorphicModel):
    name = models.CharField(max_length=30, unique=True)
    priority = models.IntegerField(unique=True)
    public = models.BooleanField()

    users = models.ManyToManyField(User, blank=True)
    permissions = models.ManyToManyField(Permission, blank=True)

    objects = StateManager()

    class Meta:
        ordering = ['priority']

    def __str__(self):
        return '%s (%s%d)' % (self.name, 'Public ' if self.public else '', self.priority)
    
    def available_to_user(self, user):
        return True == self.public 



# Generated by Django 2.2.4 on 2019-08-22 01:21

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='State',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=30, unique=True)),
                ('priority', models.IntegerField(unique=True)),
                ('public', models.BooleanField()),
                ('permissions', models.ManyToManyField(blank=True, to='auth.Permission')),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_django_user_states.state_set+', to='contenttypes.ContentType')),
                ('users', models.ManyToManyField(blank=True, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['priority'],
            },
        ),
    ]



from django.db.models import Manager, QuerySet, Q
from django.db import transaction
from functools import reduce
from polymorphic.managers import PolymorphicManager
from polymorphic.query import PolymorphicQuerySet

class StateQuerySet(PolymorphicQuerySet):
    def available_to_user(self, user):
        return [ state for state in self.all() if state.available_to_user(user) ]
    
    def get_for_user(self, user):
        states = self.available_to_user(user)
        if states:
            return states[0]
        else:
            return None

class StateManager(PolymorphicManager):
    def get_queryset(self):
        return StateQuerySet(self.model, using=self._db)
    
    def available_to_user(self, user):
        return self.get_queryset().available_to_user(user)
    
    def get_for_user(self, user):
        return self.get_queryset().get_for_user(user)